#ifndef POLYGON_H
#define POLYGON_H

/* define methods for polygon. look how the function type is defined */

/* allocate resources required for a polygon */
typedef void *(*poly_create_ctx_t)(void);
/* initialize private data structure */
typedef void (*poly_init_ctx_t)(void *,const char *);
/* deallocate the private data structure */
typedef void (*poly_destroy_ctx_t)(void *);
/* sample methods: fill inside of given polygon and get area of given polygon */
typedef void (*poly_fill_t)(void *); 
typedef double (*poly_area_t)(void *);

/* method suite for a polygon
 * I used function pointer type defined above
 */
struct polygon_ops {
    poly_create_ctx_t op_create;   /* create polygon context */
    poly_init_ctx_t op_init;   /* initialize polygon context */
    poly_destroy_ctx_t op_destroy; /* release resources for polygon context */
    poly_fill_t op_fill; /* sample method - filling polygon */
    poly_area_t op_area; /* sample method - get area */
};

typedef struct polygon_ops *polygon_ops_t;

/* generic structure for polygon */
struct polygon {
    /* this is very similar concept in OO programming language */
    void *ctx;  /* private data */
    polygon_ops_t ops;  /* function suite to manipulate the private data */
};

typedef struct polygon * polygon_t;

/* generic functions for polygon structure*/
extern polygon_t polygon_new(polygon_ops_t ops,char *spec);
extern void polygon_free(polygon_t p);
extern inline double polygon_get_area(polygon_t p);
extern inline void polygon_fill(polygon_t p);

/* sample polygon implementation: methods for triangle & rectangle. */
extern inline polygon_ops_t TRIANGLE(void);
extern inline polygon_ops_t RECTANGLE(void);

#endif /* POLYGON_H */
